import 'package:hub/components/organism_components/menu_list_view_component/menu_list_view_component_widget.dart';
import 'package:hub/flutter_flow/nav/nav.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hub/backend/schema/enums/enums.dart';
import 'package:hub/components/organism_components/menu_component/menu_component_bloc.dart';
import 'package:hub/components/organism_components/menu_staggered_view_component/menu_staggered_view_component_widget.dart';
import 'package:hub/flutter_flow/flutter_flow_theme.dart';
import 'package:hub/shared/helpers/license/license_helper.dart';
import 'package:hub/shared/services/license/license_service.dart';

class MenuComponentWidget extends StatelessWidget {
  final MenuView style;
  final MenuItem item;
  final bool expandable;
  final List<Module> menuOptions;

  const MenuComponentWidget({
    super.key,
    required this.style,
    required this.item,
    required this.expandable,
    required this.menuOptions,
  });

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<List<dynamic>>(
      stream: LicenseService().licenseStream,
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.waiting) {
          return const Center(
            child: CircularProgressIndicator(),
          );
        } else if (snapshot.hasError) {
          return Center(
            child: Text('Error: ${snapshot.error}'),
          );
        } else if (!snapshot.hasData || snapshot.data!.isEmpty) {
          return Center(
            child: Padding(
              padding: EdgeInsets.only(top: MediaQuery.of(context).size.height / 10),
              child: CircularProgressIndicator(
                  backgroundColor: FlutterFlowTheme.of(key.currentContext!).primaryBackground,
                  color: FlutterFlowTheme.of(key.currentContext!).primary),
            ),
          );
        }
        return BlocProvider(
          create: (context) => MenuBloc(
            style: style,
            item: item,
            expandable: expandable,
            menuOptions: menuOptions,
          )..add(MenuEvent()),
          child: BlocBuilder<MenuBloc, MenuState>(
            builder: (context, state) {
              if (style == MenuView.list_grid) {
                return MenuStaggeredViewComponentWidget(
                  options: state.menuEntries,
                  expandable: expandable,
                  item: item,
                  changeMenuStyle: () async {},
                  isGrid: state.isGrid,
                );
              } else if (style == MenuView.list) {
                return MenuListViewComponentWidget(
                  options: state.menuEntries,
                  expandable: expandable,
                  item: item,
                  changeMenuStyle: () async {},
                );
              }
              return const SizedBox();
            },
          ),
        );
      },
    );
  }
}
